@@ -5,6 +5,7 @@ import logging
 
 from django_filters.filters import (
     AllValuesFilter,
+    AllValuesMultipleFilter,
     BooleanFilter,
     CharFilter,
     ChoiceFilter,
@@ -16,11 +17,13 @@ from django_filters.filters import (
     DurationFilter,
     IsoDateTimeFilter,
     IsoDateTimeFromToRangeFilter,
+    MultipleChoiceFilter,
     NumberFilter,
     RangeFilter,
     TimeFilter,
     TimeRangeFilter,
     TypedChoiceFilter,
+    TypedMultipleChoiceFilter,
     UUIDFilter,
 )
 
@@ -40,16 +43,19 @@ class PropertyBaseFilterMixin():
         'lt', 'lte', 'startswith', 'istartswith', 'endswith', 'iendswith',
     ]
 
-    def __init__(self, *args, property_fld_name, **kwargs):
+    def __init__(self, *args, **kwargs):
         """Shared Constructor for Property Filters."""
         label = kwargs.get('label')
         lookup_expr = kwargs.get('lookup_expr')
 
+        # Use a different field name for properties to avoid conflicts with property_filter implementation
+        self.property_fld_name = kwargs.get('field_name')
+        kwargs['field_name'] = None
+
         if label is None:
-            label = F'{property_fld_name} [{lookup_expr}]'
+            label = F'{self.property_fld_name} [{lookup_expr}]'
             kwargs['label'] = label
 
-        self.property_fld_name = property_fld_name
         self.verify_lookup(lookup_expr)
         super().__init__(*args, **kwargs)
 
@@ -102,6 +108,39 @@ class ChoiceConvertionMixin():  # pylint: disable=too-few-public-methods
         return super()._compare_lookup_with_qs_entry(lookup_expr, new_lookup_value, new_property_value)
 
 
+class MultipleChoiceFilterMixin():  # pylint: disable=too-few-public-methods
+    """Provide filtering for multiple choice FIlter."""
+
+    def filter(self, queryset, value):
+        """Filter Multiple Choice Property Values."""
+        # If no or empty qs there is nothing to filter, leave the qs untouched
+        if not queryset or not value:
+            return queryset
+
+        result_qs = None
+
+        for sub_value in value:
+            sub_result_qs = super().filter(queryset, sub_value)
+
+            if self.conjoined:
+                if result_qs is None:
+                    # For 'AND' start from the first qs found
+                    result_qs = sub_result_qs
+
+                if sub_result_qs:
+                    result_qs = result_qs & sub_result_qs
+                else:  # Result QS empty, 'AND' will always be False, return empty qs
+                    result_qs = sub_result_qs
+            else:
+                if result_qs is None:
+                    # For 'OR' start from an empty qs
+                    result_qs = self.model.objects.none()  # pylint: disable=no-member
+
+                result_qs = result_qs | sub_result_qs
+
+        return result_qs if result_qs is not None else self.model.objects.none()
+
+
 class PropertyAllValuesFilter(ChoiceConvertionMixin, PropertyBaseFilterMixin, AllValuesFilter):
     """Adding Property Support to AllValuesFilter."""
 
@@ -123,6 +162,28 @@ class PropertyAllValuesFilter(ChoiceConvertionMixin, PropertyBaseFilterMixin, Al
         return super(AllValuesFilter, self).field
 
 
+class PropertyAllValuesMultipleFilter(
+        ChoiceConvertionMixin, MultipleChoiceFilterMixin, PropertyBaseFilterMixin, AllValuesMultipleFilter):
+    """Adding Property Support to AllValuesFilter."""
+
+    @property
+    def field(self):
+        """Filed Property to setup default choices."""
+        queryset = self.model._default_manager.distinct()  # pylint: disable=no-member,protected-access
+
+        value_list = []
+        for obj in queryset:
+            property_value = get_value_for_db_field(obj, self.property_fld_name)
+            value_list.append(property_value)
+
+        value_list = sorted(set(value_list), key=lambda x: (x is None, x))
+
+        self.extra['choices'] = [(prop, str(prop)) for prop in value_list]
+
+        # Need to Call parent's Parent since our Parent uses DB fields directly
+        return super(AllValuesMultipleFilter, self).field
+
+
 class PropertyBooleanFilter(PropertyBaseFilterMixin, BooleanFilter):
     """Adding Property Support to BooleanFilter."""
 
@@ -239,6 +300,11 @@ class PropertyIsoDateTimeFromToRangeFilter(PropertyBaseFilterMixin, IsoDateTimeF
     supported_lookups = ['range']
 
 
+class PropertyMultipleChoiceFilter(
+        ChoiceConvertionMixin, MultipleChoiceFilterMixin, PropertyBaseFilterMixin, MultipleChoiceFilter):
+    """Adding Property Support to MultipleChoiceFilter."""
+
+
 class PropertyNumberFilter(PropertyBaseFilterMixin, NumberFilter):
     """Adding Property Support to NumberFilter."""
 
@@ -265,13 +331,20 @@ class PropertyTypedChoiceFilter(ChoiceConvertionMixin, PropertyBaseFilterMixin,
     """Adding Property Support to TypedChoiceFilter."""
 
 
+class PropertyTypedMultipleChoiceFilter(
+        ChoiceConvertionMixin, MultipleChoiceFilterMixin, PropertyBaseFilterMixin, TypedMultipleChoiceFilter):
+    """Adding Property Support to TypedMultipleChoiceFilter."""
+
+
 class PropertyUUIDFilter(PropertyBaseFilterMixin, UUIDFilter):
     """Adding Property Support to UUIDFilter."""
 
     supported_lookups = ['exact']
 
 
-EXPLICIST_ONLY_FILTERS = [
+EXPLICIT_ONLY_FILTERS = [
     PropertyChoiceFilter,
     PropertyTypedChoiceFilter,
+    PropertyTypedMultipleChoiceFilter,
+    PropertyMultipleChoiceFilter,
 ]
